/*

  Copyright (C) 2001 StrmnNrmn

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

// TODO: Add a CriticalSection to stop multiple threads using the 
// Unique objects simultaneously

#include "stdafx.h"
#include "DaedalusGraphics.h"
#include "D3DRender.h"
#include "Interface\IniFile.h"

#include "GraphicsContext.h"
#include "VideoMemoryManager.h"
#include "Plugins\GraphicsPlugin_Spec.h"

#include "Utility\Profiler.h"
#include "TextureCache.h"

#include "Interface\IniFile.h"
#include "DLParser.h"

#define DAEDALUSGRAPHICS_API __declspec(dllexport)

#define DAEDALUS_GFX_PLUGIN_VERSION		0x0001

DWORD g_dwDaedalusVersion = 0;

DWORD g_dwViWidth = 320;
DWORD g_dwViHeight = 240;

#define OS_TV_PAL		0
#define OS_TV_NTSC		1
#define OS_TV_MPAL		2

BOOL g_bDropAllTextures = FALSE;

CCritSect g_CritialSection;


bool GraphicsInitialise()
{
	if(!D3DRender::Create())
		return false;

	if(!CTextureCache::Create())
		return false;

	return true;
}


/******************************************************************
  Function: InitiateGFX
  Purpose:  This function is called when the DLL is started to give
            information from the emulator that the n64 graphics
			uses. This is not called from the emulation thread.
  Input:    Gfx_Info is passed to this function which is defined
            above.
  Output:   TRUE on success
            FALSE on failure to initialise
             
  ** note on interrupts **:
  To generate an interrupt set the appropriate bit in MI_INTR_REG
  and then call the function CheckInterrupts to tell the emulator
  that there is a waiting interrupt.
*******************************************************************/ 
BOOL Graphics_InitiateGFX ()
{
	g_CritialSection.Lock();
	
	GraphicsInitialise();

	if (!DLParser_Init()) 
	{
		g_CritialSection.Unlock();
		return FALSE;
	}

	g_CritialSection.Unlock();
	return TRUE;
}

/******************************************************************
  Function: CloseDLL
  Purpose:  This function is called when the emulator is closing
            down allowing the dll to de-initialise.
  input:    none
  output:   none
*******************************************************************/ 
DAEDALUSGRAPHICS_API void __cdecl CloseDLL (void)
{
	// Do nothing - do processing in DLL_PROCESS_DETACH
	g_CritialSection.Lock();
	g_CritialSection.Unlock();
}

/******************************************************************
*******************************************************************/ 
void Graphics_ProcessDList(void)
{
	g_CritialSection.Lock();


	if (g_bDropAllTextures)
	{
		CTextureCache::Get()->DropTextures();
		g_bDropAllTextures = FALSE;
	}

	DLParser_Process();

	g_CritialSection.Unlock();
}

/******************************************************************
  Function: RomClosed
  Purpose:  This function is called when a rom is closed.
  input:    none
  output:   none
*******************************************************************/ 
void Graphics_RomClosed (void)
{
	g_CritialSection.Lock();

	if ( CTextureCache::IsAvailable() )
	{
		CTextureCache::Get()->DropTextures();
	}

	g_CritialSection.Unlock();
}

/******************************************************************
  Function: RomOpen
  Purpose:  This function is called when a rom is open. (from the 
            emulation thread)
  input:    none
  output:   none
*******************************************************************/ 

void Graphics_RomOpen (void)
{
	g_CritialSection.Lock();

	g_CritialSection.Unlock();
}


//*****************************************************************************
// Execute an arbitary command (e.g. "Take ScreenShot", "Dump DList")
// pszCommand: [in] Command name
// ppResult: [in/out] Optional auxillary data
// Returns standard HRESULT (S_OK, E_FAIL, E_OUTOFMEMORY etc)
//*****************************************************************************

HRESULT Graphics_ExecuteCommand( const CHAR * pszCommand, void ** ppResult )
{
	bool ok( false );

	g_CritialSection.Lock();

	if ( stricmp( pszCommand, DAEDALUS_GFX_SETDAEDALUSVERSION ) == 0 )
	{
		g_dwDaedalusVersion = (DWORD)*ppResult;
		ok = true;
	}
	else if ( stricmp( pszCommand, DAEDALUS_GFX_GETDAEDALUSPLUGINVERSION ) == 0 )
	{
		*ppResult = (void*)DAEDALUS_GFX_PLUGIN_VERSION;
		ok = true;
	}
	else if ( stricmp( pszCommand, DAEDALUS_GFX_DROPTEXTURES ) == 0 )
	{
		g_bDropAllTextures = TRUE;
		ok = true;
	}
	else if ( stricmp( pszCommand, DAEDALUS_GFX_DUMPTEXTURES ) == 0 )
	{
		CTextureCache::Get()->SetDumpTextures( TRUE );
		g_bDropAllTextures = TRUE;		// Drop textures so that we can re-create
		ok = true;
	}
	else if ( stricmp( pszCommand, DAEDALUS_GFX_NODUMPTEXTURES ) == 0 )
	{
		CTextureCache::Get()->SetDumpTextures( FALSE );
		ok = true;
	}
	else if ( stricmp( pszCommand, DAEDALUS_GFX_USENEWCOMBINER ) == 0)
	{
		g_bDisplayNewCombiner = (BOOL)*ppResult;
	}
	else if ( stricmp( pszCommand, DAEDALUS_GFX_DUMPDL ) == 0 )
	{
		DLParser_DumpNextDisplayList();
	}

	g_CritialSection.Unlock();
	return ok ? 0 : -1;
}







/*
void IMainWindow::SizeForDisplay()
{
	RECT rcScreen;
	RECT rcStatus;

	// This might not be the best flag to switch on - ]
	// maybe we could check g_bD3DXReady?
	if (g_bCPURunning)
	{
		SetRect(&rcScreen, 0,0, g_dwDisplayWidth, g_dwDisplayHeight);

		// Calculate window size to give desired window size...
		AdjustWindowRectEx(&rcScreen, WS_OVERLAPPEDWINDOW, TRUE, 0);

		// Add on enough space for the status bar
		GetClientRect(m_hWndStatus, &rcStatus);
		rcScreen.bottom += (rcStatus.bottom - rcStatus.top);

		SetWindowPos(m_hWnd, 0,
			rcScreen.left, rcScreen.top,
			rcScreen.right-rcScreen.left, rcScreen.bottom-rcScreen.top,
			SWP_NOMOVE|SWP_NOACTIVATE|SWP_NOZORDER);
	}
	else
	{
		rcScreen = g_rcMainWindow;

		SetWindowPos(m_hWnd, 0,
			rcScreen.left, rcScreen.top,
			rcScreen.right-rcScreen.left, rcScreen.bottom-rcScreen.top,
			SWP_NOACTIVATE|SWP_NOZORDER);
	}
}
*/
